#include "Cbmp.h"


//////////////////////////////////////////////////////////////
//// BMP Read
// constructor

// overloaded consntructor
CBmp::CBmp(char* cpFileName)
{
	LoadFile(cpFileName);
}

// destructor
CBmp::~CBmp()
{
	if(m_cpData!=NULL)
		delete m_cpData;
}


uint CBmp::GetBPP() const
{
	return m_hBitmapInfo.biBitCount;
}

uint CBmp::GetSizeOfBmp() const
{
	return m_hBitmap.bfSize;
}

uint CBmp::GetHeight() const
{
	return m_hBitmapInfo.biHeight;
}

uint CBmp::GetWidth() const
{
	return m_hBitmapInfo.biWidth;
}

uint CBmp::GetColorPlane() const
{
	return m_hBitmapInfo.biPlanes;
}

uchar* CBmp::GetBMP() const
{
	return m_cpData;
}

void CBmp::SetGapiSurface(CGapiSurface* BackBuffer)
{
	m_pBackBuffer=BackBuffer;
}

void CBmp::WriteText(char* str)
{
	FILE *dbg=NULL;
	dbg=fopen("debug.txt","a");
	fprintf(dbg,str);
	fclose(dbg);
}

void CBmp::LoadFile(char* cpFileName)
{
	FILE              *file = NULL;             // The source file
	BITMAPFILEHEADER  fileHeader;               // A header for the bitmap
	BITMAPINFOHEADER  infoHeader;
	unsigned char     *raster = NULL;           // A pointer to the raw raster data of the image
	unsigned char     *pixelData = NULL;        // A pointer to the uncompressed raster data that will be returned
	int               pixelRowLen = 0;          // The length (in unsigned chars) of a row of uncompressed pixel data
	int               pixelDataLen = 0;         // The length (in unsigned chars) of the uncompressed raster data
	BOOL              topDown = FALSE;          // Used with top down images
	int               dIndex = 0, sIndex = 0;   // Destination and source indexes in the raster/pixel data
	int               row = 0, col = 0;         // Used for looping through the raster
	int               sRowPadding = 0;          // 32-bit padding counter for source raster data
	int               dRowPadding = 0;          // 32-bit padding counter for destination pixel data
	DWORD             redMask   = 0x00007C00;   // 0111110000000000 - Default 16-bit red mask
	DWORD             greenMask = 0x000003E0;   // 0000001111100000 - Default 16-bit green mask
	DWORD             blueMask  = 0x0000001F;   // 0000000000011111 - Default 16-bit blue mask
		
	file = fopen(cpFileName, "rb");
	if(!file)
		return;

	// Read the header structures from the file
	fread(&fileHeader.bfType,2,1,file);
	fread(&fileHeader.bfSize,4,1,file);
	fread(&fileHeader.bfReserved1,2,1,file);
	fread(&fileHeader.bfReserved2,2,1,file);
	fread(&fileHeader.bfOffBits,4,1,file);

	fread( &infoHeader, sizeof( BITMAPINFOHEADER ), 1, file );
	if( infoHeader.biSizeImage == 0 )
	{
		//WriteText("line 96: \r\n");
		infoHeader.biSizeImage = fileHeader.bfSize - fileHeader.bfOffBits; 
	}
	
	// If biHeight is negative, indicating a top-down image,
	if( infoHeader.biHeight < 0 )
	{
		topDown = TRUE;
		infoHeader.biHeight = ~( infoHeader.biHeight - 1 );
	}
	
	dRowPadding  = infoHeader.biWidth % 4;
	raster  = new unsigned char[infoHeader.biSizeImage];

	pixelRowLen  = infoHeader.biWidth * 3 + dRowPadding;
	pixelDataLen = infoHeader.biHeight * pixelRowLen;
	pixelData    = new unsigned char[pixelDataLen];

	m_hBitmapInfo.biWidth=pixelRowLen;//infoHeader.biWidth;
	m_hBitmapInfo.biHeight=infoHeader.biHeight;
	m_hBitmapInfo.biBitCount=infoHeader.biBitCount;
	m_hBitmapInfo.biClrImportant=infoHeader.biClrImportant;
	m_hBitmapInfo.biClrUsed=infoHeader.biClrUsed;
    m_hBitmapInfo.biCompression=infoHeader.biCompression;
	m_hBitmapInfo.biPlanes=infoHeader.biPlanes;
    m_hBitmapInfo.biSize=infoHeader.biSize;
	m_hBitmapInfo.biSizeImage=infoHeader.biSizeImage;
	m_hBitmapInfo.biXPelsPerMeter=infoHeader.biXPelsPerMeter;
	m_hBitmapInfo.biYPelsPerMeter=infoHeader.biYPelsPerMeter;

	// If there is more data before the raster besides the headers
	if( fileHeader.bfOffBits > sizeof( BITMAPFILEHEADER ) + sizeof( BITMAPINFOHEADER ) )
	{
        if( infoHeader.biBitCount == 32 )
		{
			fread( &redMask, sizeof( DWORD ), 1, file );
			fread( &greenMask, sizeof( DWORD ), 1, file );
			fread( &blueMask, sizeof( DWORD ), 1, file );
		}
	}
	
	fseek( file, fileHeader.bfOffBits, SEEK_SET );
	fread( raster,1,infoHeader.biSizeImage,file );
	fclose( file );
	
	switch( infoHeader.biBitCount )
	{	
	case 24:
		// Three consecutive bytes (24 bits) contain the information for an individual
		// pixel.  The first byte contains the pixel's blue intensity, the second byte
		// contains the pixel's green intensity and the third byte contains the pixel's
		// red intensity.  At the end of each row of pixels there may be extra padding
		// to put the file pointer back on the 32 bit boundary (0 or 1 bytes of 0's).
		sRowPadding = dRowPadding;
		for( row = 0 ; row < infoHeader.biHeight ; row++, dIndex += dRowPadding, sIndex += sRowPadding )
		{
			for( col = 0 ; col < infoHeader.biWidth ; col++, sIndex += 3 )
			{
				pixelData[dIndex++] = raster[sIndex+2];   // Red
				pixelData[dIndex++] = raster[sIndex+1];   // Green
				pixelData[dIndex++] = raster[sIndex+0];   // Blue
			}
			
			col=0;
		}
		break;
	case 32:
		// Four consecutive bytes (32 bits) contain the information for an individual
		// pixel.  The first byte contains the pixel's blue intensity, the second byte
		// contains the pixel's green intensity, the third byte contains the pixel's red
		// intensity and the fourth byte is always 0x00.
		// There is no need for 32 bit padding.
		for( row = 0 ; row < infoHeader.biHeight ; row++, dIndex += dRowPadding )
		{
			for( col = 0 ; col < infoHeader.biWidth ; col++, sIndex += 4 )
			{
				pixelData[dIndex++] = raster[sIndex+2];   // Red
				pixelData[dIndex++] = raster[sIndex+1];   // Green
				pixelData[dIndex++] = raster[sIndex+0];   // Blue
			}
		}
		break;
	default:
		return;
	}
  
  if( !topDown )
  {
	  int x=pixelRowLen;
	  unsigned char *data=pixelData,*datacopy=pixelData;
	  unsigned char al;
	  
	  datacopy+=pixelDataLen;
	  datacopy-=x;
	  while(data<datacopy)
	  {
		  x=pixelRowLen;
		  for(;x>0;x--,data++,datacopy++)
		  {
			  al=*data;
			  *data=*datacopy;
			  *datacopy=al;   
		  }
		  datacopy-=pixelRowLen;
		  datacopy-=pixelRowLen;
	  }
  }
  
  delete[] raster;
  m_cpData=pixelData;
  
}

unsigned long CBmp::MakeRGB(uchar cR,uchar cG, uchar cB)
{
	return SymRGB(cR,cG,cB);
}

void CBmp::PutPixel( CGapiSurface* pBackBuffer, int x, int y, int r, int g, int b )
{
	GDBLTFX gdbfx;
	RECT    rcDest;
	
	if (pBackBuffer == NULL)
		return;
	
	// Initialize the DDBLTFX structure with the pixel color
	gdbfx.dwOpacity = 255;
	gdbfx.dwFillColor = (DWORD)RGB( r, g, b );
	
	// Prepare the destination rectangle as a 1x1 (1 pixel) rectangle
	SetRect( &rcDest, x, y, x+1, y+1 );
	
	// Blit 1x1 rectangle using solid color op
	pBackBuffer->Blt( &rcDest, NULL, NULL, GDBLT_COLORFILL, &gdbfx );
}

void CBmp::Display()
{
	unsigned int r,g,b,offset;
	unsigned char* bmpsurface=GetBMP();
	int width=GetWidth(),height=GetHeight();

	m_pBackBuffer->FillRect( NULL, RGB( 0, 0, 0 ), 0, NULL );
	
	for(int h=0;h<height;h++){		
		for(int w=0,x=0;w<width;w+=3,x++)
		{
			offset=(h*width)+w;
			r=*(bmpsurface+offset);
			g=*(bmpsurface+offset+1);
			b=*(bmpsurface+offset+2);
			PutPixel( m_pBackBuffer, x, h, r, g, b );
		}
	}
	
	
}